from __future__ import annotations

from typing import TYPE_CHECKING, Generic, TypeVar, cast
import uuid

T = TypeVar("T")

# Prevent circular dependency
if TYPE_CHECKING:
    from jetpack._task.client import Client


class Future(Generic[T]):
    def __init__(self, client: Client, task_id: uuid.UUID) -> None:
        self.client = client
        self.task_id = task_id

    def get(self) -> T:
        # wait_for_result returns any. Can we make this better?
        return cast(T, self.client.wait_for_result(self.task_id))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: models/runtime/describe.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from jetpack.models.runtime import cronjob_pb2 as models_dot_runtime_dot_cronjob__pb2
from jetpack.models.runtime import job_pb2 as models_dot_runtime_dot_job__pb2
from jetpack.models.core import core_pb2 as models_dot_core_dot_core__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='models/runtime/describe.proto',
  package='runtime',
  syntax='proto3',
  serialized_options=b'Z\"go.jetpack.io/axiom/models/runtime',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1dmodels/runtime/describe.proto\x12\x07runtime\x1a\x1cmodels/runtime/cronjob.proto\x1a\x18models/runtime/job.proto\x1a\x16models/core/core.proto\"\x97\x01\n\x0e\x44\x65scribeOutput\x12\x1d\n\x04meta\x18\x01 \x01(\x0b\x32\x0f.core.NonDBMeta\x12#\n\tcron_jobs\x18\x02 \x03(\x0b\x32\x10.runtime.CronJob\x12%\n\tframework\x18\x03 \x01(\x0e\x32\x12.runtime.Framework\x12\x1a\n\x04jobs\x18\x04 \x03(\x0b\x32\x0c.runtime.Job*:\n\tFramework\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x08\n\x04NONE\x10\x01\x12\t\n\x05\x46LASK\x10\x02\x12\x0b\n\x07\x46\x41STAPI\x10\x03\x42$Z\"go.jetpack.io/axiom/models/runtimeb\x06proto3'
  ,
  dependencies=[models_dot_runtime_dot_cronjob__pb2.DESCRIPTOR,models_dot_runtime_dot_job__pb2.DESCRIPTOR,models_dot_core_dot_core__pb2.DESCRIPTOR,])

_FRAMEWORK = _descriptor.EnumDescriptor(
  name='Framework',
  full_name='runtime.Framework',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NONE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FLASK', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FASTAPI', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=276,
  serialized_end=334,
)
_sym_db.RegisterEnumDescriptor(_FRAMEWORK)

Framework = enum_type_wrapper.EnumTypeWrapper(_FRAMEWORK)
UNKNOWN = 0
NONE = 1
FLASK = 2
FASTAPI = 3



_DESCRIBEOUTPUT = _descriptor.Descriptor(
  name='DescribeOutput',
  full_name='runtime.DescribeOutput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='meta', full_name='runtime.DescribeOutput.meta', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cron_jobs', full_name='runtime.DescribeOutput.cron_jobs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='framework', full_name='runtime.DescribeOutput.framework', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jobs', full_name='runtime.DescribeOutput.jobs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=123,
  serialized_end=274,
)

_DESCRIBEOUTPUT.fields_by_name['meta'].message_type = models_dot_core_dot_core__pb2._NONDBMETA
_DESCRIBEOUTPUT.fields_by_name['cron_jobs'].message_type = models_dot_runtime_dot_cronjob__pb2._CRONJOB
_DESCRIBEOUTPUT.fields_by_name['framework'].enum_type = _FRAMEWORK
_DESCRIBEOUTPUT.fields_by_name['jobs'].message_type = models_dot_runtime_dot_job__pb2._JOB
DESCRIPTOR.message_types_by_name['DescribeOutput'] = _DESCRIBEOUTPUT
DESCRIPTOR.enum_types_by_name['Framework'] = _FRAMEWORK
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DescribeOutput = _reflection.GeneratedProtocolMessageType('DescribeOutput', (_message.Message,), {
  'DESCRIPTOR' : _DESCRIBEOUTPUT,
  '__module__' : 'models.runtime.describe_pb2'
  # @@protoc_insertion_point(class_scope:runtime.DescribeOutput)
  })
_sym_db.RegisterMessage(DescribeOutput)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import time

import click
import grpc

from jetpack import _runtime as runtime
from jetpack import cron
from jetpack._task.task import Task
from jetpack.cmd import util
from jetpack.config import symbols
from jetpack.config.symbols import Symbol

_using_new_cli = False


def is_using_new_cli() -> bool:
    """
    for legacy uses, we keep old cli. This function disables that logic to ensure
    we don't run the cli command twice.
    """
    return _using_new_cli


@click.group()
def cli() -> None:
    global _using_new_cli
    _using_new_cli = True


@click.command(help="Executes jetpack task")
@click.option("--entrypoint", required=True)
@click.option("--exec-id", required=True)
@click.option("--qualified-symbol", required=True)
@click.option("--encoded-args", default="")
def exec_task(
    entrypoint: str, exec_id: str, qualified_symbol: str, encoded_args: str
) -> None:
    util.load_user_entrypoint(entrypoint)
    func = symbols.get_symbol_table()[Symbol(qualified_symbol)]
    Task(func).exec(exec_id, encoded_args)


@click.command(help="Registers jetpack functions with runtime")
@click.option("--entrypoint", required=True)
def register(entrypoint: str) -> None:
    util.load_user_entrypoint(entrypoint)
    # For now, we try a few times to register with runtime. Once the runtime
    # becomes a sidecar, we can remove this.
    tries = 3
    for i in range(tries):
        try:
            runtime.set_cron_jobs(cron.get_jobs())
        except grpc.RpcError as rpc_error:
            if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:
                if i == tries - 1:
                    # no more tries, just give up.
                    raise rpc_error
                print("runtime is not available. Sleep(5) and try again")
                time.sleep(5)
            else:
                raise rpc_error


cli.add_command(exec_task)
cli.add_command(register)
